﻿using Microsoft.Win32;
using STM.EfHubService.DataAccess;
using STM.EfHubService.DataAccess.Entities.Ais;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Xml;

namespace STM.EfHubService.AdminUI
{
    public class AisViewModel : ViewModelBase
    {
        public AisViewModel()
        {
            LoadAisData();
        }

        private ObservableCollection<Ship> _ships;
        public ObservableCollection<Ship> Ships
        {
            get
            {
                return _ships;
            }
            set
            {
                if (_ships == value)
                    return;

                _ships = value;
                OnPropertyChanged(() => Ships);
            }
        }

        private Ship _selectedShip;
        public Ship SelectedShip
        {
            get
            {
                return _selectedShip;
            }
            set
            {
                if (_selectedShip == value)
                    return;

                _selectedShip = value;
                OnPropertyChanged(() => SelectedShip);
                AddCommand.RaiseCanExecuteChanged();
                UpdateCommand.RaiseCanExecuteChanged();
                RemoveCommand.RaiseCanExecuteChanged();
            }
        }

        private DelegateCommand _addCommand;
        public DelegateCommand AddCommand
        {
            get
            {
                return _addCommand ??
                    (_addCommand = new DelegateCommand(ExecuteAddCommand, CanExecuteAddCommand));
            }
        }

        public bool CanExecuteAddCommand(object parameter)
        {
            return true;
        }

        public async void ExecuteAddCommand(object parameter)
        {
            var dlg = new AisShipDialog();
            dlg.ViewModel.IsUpdate = false;
            dlg.ViewModel.SelectedShip = new Ship
            {
                Latitude = 59.690000m,
                Longitude = 19.720000m,
                Sog = 10,
                UpdatedUtc = DateTime.Parse("2021-01-01")
            };

            if (dlg.ShowDialog() == true)
            {
                AddShip(dlg.ViewModel.SelectedShip);
            }
        }

        private DelegateCommand _removeCommand;
        public DelegateCommand RemoveCommand
        {
            get
            {
                return _removeCommand ??
                    (_removeCommand = new DelegateCommand(ExecuteRemoveCommand, CanExecuteRemoveCommand));
            }
        }

        public bool CanExecuteRemoveCommand(object parameter)
        {
            return SelectedShip != null;
        }

        public async void ExecuteRemoveCommand(object parameter)
        {
            if (MessageBox.Show($"Are you sure that you want to delete the ship with MMSI: {SelectedShip.Mmsi} IMO: {SelectedShip.Imo} Name: {SelectedShip.Name}", "Delete ship", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
            {
                DeleteShip(SelectedShip);
            }
        }

        private DelegateCommand _updateCommand;
        public DelegateCommand UpdateCommand
        {
            get
            {
                return _updateCommand ??
                    (_updateCommand = new DelegateCommand(ExecuteUpdateCommand, CanExecuteUpdateCommand));
            }
        }

        public bool CanExecuteUpdateCommand(object parameter)
        {
            return SelectedShip != null;
        }

        public async void ExecuteUpdateCommand(object parameter)
        {
            var dlg = new AisShipDialog();
            dlg.ViewModel.IsUpdate = true;
            dlg.ViewModel.SelectedShip = SelectedShip;

            if (dlg.ShowDialog() == true)
            {
                UpdateShip(SelectedShip);
            }
        }


        private void LoadAisData()
        {
            //using (var aisContext = new AisDbContext())
            //{
            //    Ships = new ObservableCollection<Ship>(aisContext.Ships.ToList());
            //}
        }

        private void UpdateShip(Ship ship)
        {
            try
            {
                //using (var aisContext = new AisDbContext())
                //{
                //    var dbShip = aisContext.Ships.FirstOrDefault(x => x.Mmsi == ship.Mmsi);
                //    dbShip.Imo = ship.Imo;
                //    dbShip.Name = ship.Name;
                //    dbShip.Latitude = ship.Latitude;
                //    dbShip.Longitude = ship.Longitude;
                //    dbShip.Sog = ship.Sog;
                //    dbShip.UpdatedUtc = ship.UpdatedUtc;

                //    aisContext.SaveChanges();
                //}

                LoadAisData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
}

        private void AddShip(Ship ship)
        {
            try
            { 
                //using (var aisContext = new AisDbContext())
                //{
                //    aisContext.Ships.Add(ship);
                //    aisContext.SaveChanges();
                //}

                LoadAisData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void DeleteShip(Ship ship)
        {
            try
            {
                //using (var aisContext = new AisDbContext())
                //{
                //    var dbShip = aisContext.Ships.FirstOrDefault(x => x.Mmsi == ship.Mmsi);
                //    aisContext.Ships.Remove(dbShip);
                //    aisContext.SaveChanges();
                //}

                LoadAisData();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
    }
}