﻿using STM.EfHubService.DataAccess.Entities;
using STM.EfHubService.DataAccess.Entities.Ais;
using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.Entity.ModelConfiguration.Conventions;
using System.Linq;

namespace STM.EfHubService.DataAccess
{
    public class EfHubServiceDbContext : DbContext
    {
        public DbSet<Service> Service { get; set; }
        public DbSet<Settings> Settings { get; set; }
        public DbSet<FlowArea> FlowArea { get; set; }
        public DbSet<FlowOccasion> FlowOccasion { get; set; }
        public DbSet<FlowAreaShipPassage> FlowAreaShipPassage { get; set; }
        public DbSet<UploadedRtz> UploadedRtz { get; set; }

        public EfHubServiceDbContext()
            : base("EfHubServiceConnectionString")
        {
            var ensureDLLIsCopied = System.Data.Entity.SqlServer.SqlProviderServices.Instance;

            //Database.SetInitializer(new MigrateDbToLatestInitializerConnString<EfHubServiceDbContext,
            //    Migrations.EfHubServiceServiceConfiguration>(Database.Connection.ConnectionString));
        }

        protected override void OnModelCreating(DbModelBuilder modelBuilder)
        {
            modelBuilder.Conventions.Remove<PluralizingTableNameConvention>();
            modelBuilder.Entity<Ship>()
                .Property(p => p.Latitude)
                .HasPrecision(10, 6);
            modelBuilder.Entity<Ship>()
                .Property(p => p.Longitude)
                .HasPrecision(10, 6);
        }

        public override int SaveChanges()
        {
            var entities = ChangeTracker.Entries()
                .Where(x => x.Entity is EntityBase
                && (x.State == EntityState.Added
                || x.State == EntityState.Modified));

            var timestamp = DateTime.UtcNow;
            timestamp = DateTime.SpecifyKind(timestamp, DateTimeKind.Local);

            foreach (var entity in entities)
            {
                if (entity.Entity is EntityBase)
                {
                    if (entity.State == EntityState.Added)
                    {
                        if (((EntityBase)entity.Entity).Created == DateTime.MinValue)
                            ((EntityBase)entity.Entity).Created = timestamp;
                    }
                    if (entity.State == EntityState.Modified)
                    {
                        ((EntityBase)entity.Entity).Updated = timestamp;
                    }
                }
            }

            return base.SaveChanges();
        }


    }
}